package com.topcoder.message.im.server;

import java.net.Socket;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.topcoder.message.im.ConnectionInitializationException;
import com.topcoder.message.im.ConnectionStateException;
import com.topcoder.message.im.Message;
import com.topcoder.message.im.MessageProcessor;
import com.topcoder.message.im.connection.GenericSocketConnection;

/**
 * The MessageServer class implements a chat server.
 * 
 * The functionality supported by the chat server is the following:
 * 
 * - logon (see LogonMessage)
 * - logoff (see LogoffMessage)
 * - create user (see ProfileMessage)
 * - remove user (see ProfileMessage)
 * - update user profile (see ProfileMessage)
 * - get user profile (see ProfileMessage)
 * - send message (see TextMessage)
 * 
 * The server opens a generic connection using as listen port the port given by the user
 * (either through the config file or as constructor parameter). The server implements the
 * MessageProcessor interface so it can take care of the messages that are comming from the users.
 * 
 * The server keeps internally three maps, one with the logged users (maps usernames to Socket connections)
 * and one with the profiles of the users (not necessarily logged on) and one with the contacts for each
 * user. Note that this version does not provide persistence for the profiles. Future versions can simply 
 * extend this class and provide profile persistence services (by overriding startServer and 
 * stopServer for example to load/save the profile map to a persistent storage). Same thing applies
 * for contact information. 
 */
public class MessageServer implements MessageProcessor {
	
	/**
	 * The Configuration Manager namespace used by this class.
	 * 
	 */
	public static final String NAMESPACE = "com.topcoder.message.im.server.MessageServer";
	
	/**
	 * The default config manager file. This file is loaded only if the 
	 * config manager namespace is not created when the component is instantiated 
	 * for the first time. This gives the flexibility to use other config manager 
	 * file locations or formats, essential in development of applications using many 
	 * different components, with different configuration files, to avoid the nightmare 
	 * of having a lot of different config files, spread everywhere. 
	 */
	private static final String DEFAULT_CONFIG_FILE = "com/topcoder/message/im/server/MessageServer.properties";
	
	/**
	 * The listening port. Set in the constructors.
	 */
	private int port = 0;
	
	/**
	 * The connection on which the server is receiving and sending messages.
	 * A GenericConnection should be used here. The generic connection should be
	 * kept in synch with the clients map. When a user logs on, it should be
	 * registered as recipient in the generic connection. When the use logs
	 * off, it should be unregistered.
	 * 
	 * The server uses this generic connection to send messages to user and to
	 * dispatch messages sent by users to other users.
	 */
	private GenericSocketConnection serverConnection = null;
	
	/**
	 * A map with the logged on clients.
	 * This map is updated when logon/logoff messages are sent.
	 * It maps user names to sockets.
	 * The purpose of this map is to keep track of the address and port of
	 * each connected user.
	 */
	private Map clients = new HashMap();
	
	/**
	 * Map storing the profiles of the users.
	 * Among other things, this map is user for authentication, since the
	 * UserProfile contains the password.
	 * The map is modified by create user messages, update profile messages,
	 * remove user messages. The get profile message can be used to find
	 * information about a user.
	 */
	private Map profiles = new HashMap();
	
	/**
	 * Map storing the contacts of the users.
	 * Among other things, this map is user for authorization for the get profile operation
	 * (a user can get another users profile according to the visibility).
	 * The map is modified by create user messages, update profile messages,
	 * remove user messages. The get profile message can be used to find
	 * information about a user.
	 */
	private Map contacts = new HashMap();
	
	/**
	 * Constructor.
	 * 
	 * Reads the listening port from the config file.
	 * If the namespace is not loaded it is loaded automatically.
	 * 
	 * @throws MessageServerException wraps any configuration manager exception or
	 * signals missing propery
	 */
	public MessageServer() throws MessageServerException {		
	}
	
	/**
	 * Constructor using a custom namespace.
	 * 
	 * Reads the listening port from the config file.
	 * 
	 * @param namespace the custom namespace
	 * 
	 * @throws MessageServerException wraps any configuration manager exception or
	 * signals missing propery
	 * @throws NullPointerException if the argument is null 
	 */
	public MessageServer(String namespace) throws NullPointerException {		
	}
	
	/**
	 * Constructor with given port.
	 * 
	 * @param listenPort the port
	 * @throws IllegalArgumentException if the port is negative or zero
	 */
	public MessageServer(int listenPort) throws IllegalArgumentException {	
	}
	
	/**
	 * Starts the server by opening the connection (simply calls openConnection of serverConnection).
	 * The code will pass this instance as message processor.
	 * If connection is already open, nothing is performed and ConnectionStateException is thrown.</p>	
	 * 
	 * @exception ConnectionStateException if connection is already open
	 * @exception ConnectionInitializationException if initialization fails
	 */ 
	public void startServer() throws ConnectionStateException, ConnectionInitializationException {		
	}
	
	/**
	 * Returns the listening port
	 * 
	 * @return the listening port.
	 */
	public int getPort() {
		return port;
	}
	
	/**
	 * Sets the listening port.
	 * If the server is already started an exception is thrown
	 * 
	 * @param port the port to set
	 * 
	 * @throws IllegalArgumentException if the port is negative or zero
	 * @throws ConnectionStateException if the server is started
	 */
	public void setPort(int port) throws IllegalArgumentException, ConnectionStateException {
		this.port = port;
	}
	
	/**
	 * Stops the server by closing the connection (simply calls openConnection of serverConnection).
	 */ 
	public void stopServer() { 
	}
	
	/**
	 * Returns whether the server is started.
	 * Delegated to serverConnection.isConnectionOpen.
	 * 
	 * @return whether the server is started
	 */
	public boolean isStarted() {
		return false;
	}
	
	/**
	 * Returns the set with the logged on clients (user names).
	 * If the server is not started, an empty set should be returned.
	 * 
	 * @return the logged on clients (user names - Strings) 
	 */
	public Set getLoggedClients() {
		return null;
	}
	
	/**
	 * This is the most important method of this class since this handles all
	 * the logic of the chat server. The following types of messages are handled:
	 * 
	 * logon: Comes as a LogonMessage instance with username and password set.
	 * Looks at the user profile for the username and checks the password.
	 * Sends back the same LogonMessage instance to the client. If the logon
	 * is not succesful (bad user, bad password), the error message is set accordingly.
	 * Also updates the clients map and registers the sender in the server connection.
	 * 
	 * logoff: Comes as a LogoffMessage. Using the username it removes the
	 * clients map entry and unregisters the sender from the server connection
	 * 
	 * create user: Comes as a ProfileMessage with create user command.
	 * The user profile and contacts are put in the two maps.
	 * 
	 * remove user: Comes as a ProfileMessage with remove user command.
	 * The user profile and contacts are removed from the two maps.
	 * 
	 * get user profile: Comes as a ProfileMessage with get profile command.
	 * The profile and contacts for username are located and returned in the
	 * same message. Note that the visibility of the desired profile is checked.
	 * 
	 * update profile: Comes as a ProfileMessage with update profile command.
	 * The user profile and contacts are updated in the two maps.
	 * 
	 * send message: Comes as a TextMessage. It simply calls the serverConnection
	 * sendMessage that does all the dispatching for us.
	 * 
	 * direct connection reques. Comes as a DirectConnectionMessage. It sends
	 * the same message back completed with the address and port for the given user.
	 * 
	 * @param message the received message
	 * @param socket the socket on which the message arrives
	 */
	public void processMessage(Message message, Socket socket) {
		// TODO Auto-generated method stub
		
	}
	
}
